/*!
  @file
  @author Shin'ichiro Nakaoka
*/

#ifndef CNOID_UTIL_TRIANGLE_MESH_SHAPER_H_INCLUDED
#define CNOID_UTIL_TRIANGLE_MESH_SHAPER_H_INCLUDED

#include "VRMLNodes.h"
#include <boost/signal.hpp>
#include "exportdecl.h"

namespace cnoid
{
    class TMSImpl;
    
    class CNOID_EXPORT TriangleMeshShaper
    {
      public:

        TriangleMeshShaper();
        ~TriangleMeshShaper();

        void setDivisionNumber(int n);
        void setNormalGenerationMode(bool on);
        VRMLNodePtr apply(VRMLNodePtr topNode);
        SFNode getOriginalGeometry(VRMLShapePtr shapeNode);
        void defaultTextureMapping(VRMLShape* shapeNode);
        
        boost::signal<void(const std::string& message)> sigMessage;
        
        bool convertBox(VRMLBox* box, VRMLIndexedFaceSetPtr& triangleMesh);

      private:
        TMSImpl* impl;

        void defaultTextureMappingFaceSet(VRMLIndexedFaceSet* triangleMesh);
        void defaultTextureMappingElevationGrid(VRMLElevationGrid* grid, VRMLIndexedFaceSet* triangleMesh);
        void defaultTextureMappingBox(VRMLIndexedFaceSet* triangleMesh);
        void defaultTextureMappingCone(VRMLIndexedFaceSet* triangleMesh);
        void defaultTextureMappingCylinder(VRMLIndexedFaceSet* triangleMesh);
        void defaultTextureMappingSphere(VRMLIndexedFaceSet* triangleMesh, double radius);
        void defaultTextureMappingExtrusion(VRMLIndexedFaceSet* triangleMesh, VRMLExtrusion* extrusion );
        int faceofBox(SFVec3f* point);
        int findPoint(MFVec2f& points, SFVec2f& target);
        double calcangle(SFVec3f& point);
    };

    enum { LEFT, TOP, FRONT, BOTTOM, RIGHT, BACK };

};

#endif

